@@ -1,39 +1,45 @@
 package Mojolicious::Plugin::HostMeta;
 use Mojo::Base 'Mojolicious::Plugin';
+use Mojo::UserAgent;
 use Mojo::Headers;
+use Mojo::JSON;
 use Mojo::Util qw/quote/;
+use Mojo::IOLoop;
 
-our $VERSION = '0.16_1';
 
-our $WK_PATH = '/.well-known/host-meta';
+our $VERSION = 0.04;
+
+
+my $WK_PATH = '/.well-known/host-meta';
+my $UA_NAME = __PACKAGE__ . ' v' . $VERSION;
 
 
 # Register plugin
 sub register {
-  my ($plugin, $app, $param) = @_;
+  my ($plugin, $mojo, $param) = @_;
 
   $param ||= {};
 
   # Load parameter from Config file
-  if (my $config_param = $app->config('HostMeta')) {
-    $param = { %$param, %$config_param };
+  if (my $config_param = $mojo->config('HostMeta')) {
+    $param = { %$config_param, %$param };
   };
 
   # Get helpers object
-  my $helpers = $app->renderer->helpers;
+  my $helpers = $mojo->renderer->helpers;
 
   # Load Util-Endpoint/Callback if not already loaded
   foreach (qw/Endpoint Callback/) {
-    $app->plugin("Util::$_") unless exists $helpers->{ lc $_ };
-  };
-
-  # Load XML if not already loaded
-  unless (exists $helpers->{render_xrd}) {
-    $app->plugin('XRD');
+    $mojo->plugin("Util::$_") unless exists $helpers->{ lc $_ };
   };
 
   # Set callbacks on registration
-  $app->callback(fetch_hostmeta => $param);
+  $mojo->callback(['hostmeta_fetch'] => $param);
+
+  # Load XML if not already loaded
+  unless (exists $helpers->{render_xrd}) {
+    $mojo->plugin('XRD');
+  };
 
   # Get seconds to expiration
   my $seconds = (60 * 60 * 24 * 10);
@@ -42,14 +48,14 @@ sub register {
   };
 
   # Create new hostmeta document
-  my $hostmeta = $app->new_xrd;
+  my $hostmeta = $mojo->new_xrd;
   $hostmeta->extension( -HostMeta );
 
   # Get host information on first request
-  $app->hook(
+  $mojo->hook(
     prepare_hostmeta =>
       sub {
-	my ($c, $hostmeta) = @_;
+	my ($c, $xrd_ref) = @_;
 	my $host = $c->req->url->to_abs->host;
 
 	# Add host-information to host-meta
@@ -58,7 +64,7 @@ sub register {
     );
 
   # Establish 'hostmeta' helper
-  $app->helper(
+  $mojo->helper(
     hostmeta => sub {
       my $c = shift;
 
@@ -77,7 +83,7 @@ sub register {
     });
 
   # Establish /.well-known/host-meta route
-  my $route = $app->routes->route( $WK_PATH );
+  my $route = $mojo->routes->route( $WK_PATH );
 
   # Define endpoint
   $route->endpoint('host-meta');
@@ -104,7 +110,7 @@ sub register {
       };
 
       # Serve host-meta document
-      return $c->helpers->render_xrd(
+      return $c->render_xrd(
 	_serve_hostmeta( $c, $hostmeta )
       );
     });
@@ -116,9 +122,6 @@ sub _fetch_hostmeta {
   my $c    = shift;
   my $host = lc shift;
 
-  # Trim tail
-  pop while @_ && !defined $_[-1];
-
   # Get headers
   my $header = {};
   if ($_[0] && ref $_[0] && ref($_[0]) eq 'HASH') {
@@ -126,7 +129,7 @@ sub _fetch_hostmeta {
   };
 
   # Check if security is forced
-  my $secure = defined $_[-1] && $_[-1] eq '-secure' ? pop : 0;
+  my $secure = $_[-1] && $_[-1] eq '-secure' ? pop : 0;
 
   # Get callback
   my $cb = pop if ref($_[-1]) && ref($_[-1]) eq 'CODE';
@@ -141,12 +144,9 @@ sub _fetch_hostmeta {
   my $rel;
   $rel = shift if $_[0] && ref($_[0]) eq 'ARRAY';
 
-  # Helpers proxy
-  my $h = $c->helpers;
-
   # Callback for caching
-  my ($xrd, $headers) = $h->callback(
-    fetch_hostmeta => $host
+  my ($xrd, $headers) = $c->callback(
+    hostmeta_fetch => $host
   );
 
   # HostMeta document was cached
@@ -171,7 +171,7 @@ sub _fetch_hostmeta {
 
   # Non-blocking
   if ($cb) {
-    return $h->get_xrd(
+    return $c->get_xrd(
       $path => $header => sub {
 	my ($xrd, $headers) = @_;
 	if ($xrd) {
@@ -199,7 +199,7 @@ sub _fetch_hostmeta {
   };
 
   # Blocking
-  ($xrd, $headers) = $h->get_xrd( $path => $header );
+  ($xrd, $headers) = $c->get_xrd( $path => $header );
 
   # No host-meta found
   return unless $xrd;
@@ -228,11 +228,8 @@ sub _serve_hostmeta {
   my $c   = shift;
   my $xrd = shift;
 
-  # Delete tail
-  pop while @_ && !defined $_[-1];
-
   # Ignore security flag
-  pop if defined $_[-1] && $_[-1] eq '-secure';
+  pop if $_[-1] && $_[-1] eq '-secure';
 
   # Ignore header information
   shift if $_[0] && ref($_[0]) && ref($_[0]) eq 'HASH';
@@ -269,7 +266,7 @@ sub _serve_hostmeta {
   };
 
   # Clone hostmeta reference
-  $xrd = $c->helpers->new_xrd( $xrd->to_string );
+  $xrd = $c->new_xrd( $xrd->to_xml );
 
   # Emit 'before_serving_hostmeta' hook
   $plugins->emit_hook(
@@ -295,13 +292,13 @@ __END__
 
 =head1 NAME
 
-Mojolicious::Plugin::HostMeta - Serve and Retrieve Host-Meta Documents
+Mojolicious::Plugin::HostMeta - Serve and Retrieve Host-Meta documents
 
 
 =head1 SYNOPSIS
 
   # Mojolicious
-  $app->plugin('HostMeta');
+  $self->plugin('HostMeta');
 
   # Mojolicious::Lite
   plugin 'HostMeta';
@@ -309,10 +306,10 @@ Mojolicious::Plugin::HostMeta - Serve and Retrieve Host-Meta Documents
   # Serves XRD or JRD from /.well-known/host-meta
 
   # Blocking requests
-  print $c->hostmeta('gmail.com')->link('lrrd');
+  print $self->hostmeta('gmail.com')->link('lrrd');
 
   # Non-blocking requests
-  $c->hostmeta('gmail.com' => sub {
+  $self->hostmeta('gmail.com' => sub {
     print shift->link('lrrd');
   });
 
@@ -320,12 +317,13 @@ Mojolicious::Plugin::HostMeta - Serve and Retrieve Host-Meta Documents
 =head1 DESCRIPTION
 
 L<Mojolicious::Plugin::HostMeta> is a Mojolicious plugin to serve and
-request C<well-known> L<Host-Meta|https://tools.ietf.org/html/rfc6415>
+request "well-known" L<Host-Meta|https://tools.ietf.org/html/rfc6415>
 documents.
 
+
 =head1 METHODS
 
-=head2 register
+=head2 C<register>
 
   # Mojolicious
   $app->plugin(HostMeta => {
@@ -339,24 +337,23 @@ Called when registering the plugin.
 Accepts one optional parameter C<expires>, which is the number
 of seconds the served host-meta should be cached by the fetching client.
 Defaults to 10 days.
-All parameters can be set either as part of the configuration
-file with the key C<HostMeta> or on registration
-(that can be overwritten by configuration).
+This parameter can be set either on registration or
+as part of the configuration file with the key C<HostMeta>.
 
 
 =head1 HELPERS
 
-=head2 hostmeta
+=head2 C<hostmeta>
 
   # In Controller:
-  my $xrd = $c->hostmeta;
-  $xrd = $c->hostmeta('gmail.com');
-  $xrd = $c->hostmeta('sojolicio.us' => ['hub']);
-  $xrd = $c->hostmeta('sojolicio.us', { 'X-MyHeader' => 'Fun' } => ['hub']);
-  $xrd = $c->hostmeta('gmail.com', -secure);
+  my $xrd = $self->hostmeta;
+  $xrd = $self->hostmeta('gmail.com');
+  $xrd = $self->hostmeta('sojolicio.us' => ['hub']);
+  $xrd = $self->hostmeta('sojolicio.us', { 'X-MyHeader' => 'Fun' } => ['hub']);
+  $xrd = $self->hostmeta('gmail.com', -secure);
 
   # Non blocking
-  $c->hostmeta('gmail.com' => ['hub'] => sub {
+  $self->hostmeta('gmail.com' => ['hub'] => sub {
     my $xrd = shift;
     # ...
   }, -secure);
@@ -380,19 +377,15 @@ only over C<https> without redirections.
 This method can be used in a blocking or non-blocking way.
 For non-blocking retrieval, pass a callback function as the
 last argument before the optional C<-secure> flag to the method.
-As the first passed response is the L<XML::Loy::XRD>
-document, you have to use an offset of C<0> in
-L<begin|Mojo::IOLoop::Delay/begin> for parallel requests using
-L<Mojo::IOLoop::Delay>.
 
 
 =head1 CALLBACKS
 
-=head2 fetch_hostmeta
+=head2 hostmeta_fetch
 
   # Establish a callback
-  $app->callback(
-    fetch_hostmeta => sub {
+  $mojo->callback(
+    hostmeta_fetch => sub {
       my ($c, $host) = @_;
 
       my $doc = $c->chi->get("hostmeta-$host");
@@ -420,16 +413,14 @@ helper or on registration.
 
 This can be used for caching.
 
-Callbacks may be changed for non-blocking requests.
-
 
 =head1 HOOKS
 
 =head2 prepare_hostmeta
 
-  $app->hook(prepare_hostmeta => sub {
-    my ($c, $xrd) = @_;
-    $xrd->link(permanent => '/perma.html');
+  $mojo->hook(prepare_hostmeta => sub {
+    my ($c, $hostmeta) = @_;
+    $hostmeta->link(permanent => '/perma.html');
   };
 
 This hook is run when the host's own host-meta document is
@@ -440,9 +431,9 @@ This hook is only emitted once for each subscriber.
 
 =head2 before_serving_hostmeta
 
-  $app->hook(before_serving_hostmeta => sub {
-    my ($c, $xrd) = @_;
-    $xrd->link(lrdd => './well-known/host-meta');
+  $mojo->hook('before_serving_hostmeta' => sub {
+    my ($c, $hostmeta) = @_;
+    $hostmeta->link(lrdd => './well-known/host-meta');
   };
 
 This hook is run before the host's own host-meta document is
@@ -454,13 +445,13 @@ for each request.
 
 =head2 after_fetching_hostmeta
 
-  $app->hook(
+  $mojo->hook(
     after_fetching_hostmeta => sub {
       my ($c, $host, $xrd, $headers) = @_;
 
       # Store in cache
       my $chi = $c->chi;
-      $chi->set("hostmeta-$host" => $xrd->to_string);
+      $chi->set("hostmeta-$host" => $xrd->to_xml);
       $chi->set("hostmeta-$host-headers" => $headers->to_string);
     }
   );
@@ -512,14 +503,12 @@ L<Mojolicious::Plugin::XRD>.
 
   https://github.com/Akron/Mojolicious-Plugin-HostMeta
 
-This plugin is part of the L<Sojolicious|http://sojolicio.us> project.
-
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011-2014, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2011-2013, L<Nils Diewald|http://nils-diewald.de/>.
 
 This program is free software, you can redistribute it
-and/or modify it under the terms of the Artistic License version 2.0.
+and/or modify it under the same terms as Perl.
 
 =cut
